#!/bin/bash

# Name of executable to test
c2x=${1:-c2x}

diff=diff
numdiff="numdiff"
tol1='-a 2e-6'
tol2='-r 1e-5'
tol="$tol1 $tol2"

passes=0
fails=0
total_passes=0
total_fails=0

pass(){
    if $diff -w $1 $2 > /dev/null 2>&1
    then
	echo pass
	passes=$((passes+1))
    else
	echo Fail
	fails=$((fails+1))
    fi
}

npass(){
    if $numdiff $tol $1 $2 > /dev/null 2>&1
    then
	echo pass
	passes=$((passes+1))
    else
	echo Fail
	fails=$((fails+1))
    fi
}

# Test ability to expand XSF file with different prim and conv coords

echo -n 'XSF expansion: '
$c2x --cell SiC8.xsf > out.cell
pass out.cell SiC8.cell

# Test ability to expand CIF file with symmetry operations to expand

echo -n 'CIF expansion: '
$c2x --cell SiC8.cif | sed -e '/^%block SYMMETRY_OPS/,$d' | \
    sed -e '$d' > out.cell
pass out.cell SiC8.cell

echo -n 'CIF expansion (cartesians): '
$c2x --cell SiC8_cart.cif | sed -e '/^%block SYMMETRY_OPS/,$d' | \
    sed -e '$d' > out.cell
pass out.cell SiC8.cell


# Test ability to read pdb files

echo -n 'PDB read: '
$c2x --cell SiC8.pdb out.cell
pass out.cell SiC8.cell

echo -n 'PDBn read: '
$c2x --cell SiC8n.pdb out.cell
pass out.cell SiC8.cell

# Test ability to read SHELX97 files

echo -n 'SHELX97 read: '
$c2x --cell SiC8.res | sed '/^#/d' > out.cell
pass out.cell SiC8.cell

# Test ability to read cell files

echo -n 'Cell read: '
$c2x --cell SiC8.cell out.cell
pass out.cell SiC8.cell

echo -n 'Cell_abs read: '
$c2x --cell SiC8_abs.cell out.cell
npass out.cell SiC8.cell

echo -n 'Cell_abc read/write: '
$c2x --cell_abc SiC8.cell out.cell
$c2x --cell out.cell out2.cell
npass out2.cell SiC8.cell

echo -n 'Cell_abs read/write (atomic units): '
$c2x --cell_abs -u SiC8.cell out.cell
$c2x --cell out.cell out2.cell
npass out2.cell SiC8.cell

echo -n 'Cell_abc_abs read/write (atomic units): '
$c2x --cell_abc_abs -u SiC8.cell out.cell
$c2x --cell out.cell out2.cell
npass out2.cell SiC8.cell

echo -n 'Cell read with subspecies: '
$c2x --cell_abs Methane.cell out.cell
npass out.cell Methane.cell

echo -n 'Cell read with arithetic: '
$c2x --cell_abc SiC_4Ha.cell out.cell
npass out.cell SiC_4H.cell

echo -n 'Onetep read: '
$c2x -1 --cell SiC8.one out.cell
npass SiC8.cell out.cell

echo -n 'FDF read: '
$c2x --cell FeO_inc.fdf out.cell
npass FeO.cell out.cell
rm out.cell

echo -n 'FDF write: '
$c2x --fdf FeO_inc.fdf out.fdf
$c2x --cell out.fdf out.cell
npass FeO.cell out.cell
rm out.fdf

echo -n 'Crystal read : '
$c2x --cell -n calcite.12 out.cell
npass calcite.cell out.cell

echo -n 'Crystal read (2): '
$c2x --cell -n quartz_a.d12 out.cell
npass quartz_a.cell out.cell

echo -n 'Abinit read: '
$c2x --cell FeO.in out.cell
npass FeO_kpt.cell out.cell

echo -n 'Abinit read (2): '
$c2x --cell -n calcite.in out.cell
npass calcite.cell out.cell

echo -n 'QE read: '
$c2x --cell -Q2 NaCl.in out.cell
npass NaCl.cell out.cell

echo -n 'QE read (2): '
$c2x -X='(5.69,0,0)(0,5.69,0)(0,0,5.69)' -Q2 --cell NaCl_2.in out.cell
# remove k-points
head -n -3 NaCl.cell > out2.cell
npass out2.cell out.cell

# Python output test

echo -n 'Pya write: '
$c2x --pya SiC8.cell out.pya
npass out.pya SiC8.pya
rm out.pya

# Test ability to find primitive cells

echo -n 'Internal primitive cell finder: '
$c2x -P --cell_abc SiC8.cell out.cell
npass out.cell SiC2_abc.cell

echo -n 'Manual transformations: '
$c2x -x='(.5,.5,0)(-.5,.5,0)(0,0,1)' --cell SiC8.cell  out.cell
$c2x -X='(4.36,0,0)(0,4.36,0)(0,0,4.36)' --cell out.cell  out2.cell
npass SiC8.cell out2.cell

echo -n 'Failure star: '
$c2x -f -M=2,2,2,.5,.5,.5 --null SiC8.cell 2> out.dat
cat > out2.dat <<EOF
First failure star, integral -1.000000
Typical vector: (0.000000,0.000000,8.720000), length**2=76.038400
EOF
npass out.dat out2.dat

echo
echo "Cell tests complete."
echo "Passed: $passes"
echo "Failed: $fails"
echo

total_passes=$((total_passes+passes))
total_fails=$((total_fails+fails))

passes=0
fails=0

echo -n 'SPGlib international symbol: '
$c2x --int SiC8.cell 2> out.dat
cat > out2.dat <<EOF
International symmetry is F-43m
EOF
pass out.dat out2.dat

echo -n 'SPGlib international symbol (2): '
$c2x --int SiC2_abc.cell 2> out.dat
pass out.dat out2.dat

echo -n 'SPGlib primitive cell finder: '
$c2x --primitive --cell_abc SiC8.cell out.cell
npass out.cell SiC2_abc.cell

# Skip next test as spglib's output is not stable
if false
then
echo -n 'SPGlib symmetry ops: '
$c2x --list SiC_4H.cell 2> out.dat
cat > out2.dat <<EOF
12 symmetry operations found.
Identity
 6  axis along (-0.000,-0.000, 1.000) through (0.333333,0.333333,0.000000)
 3  axis along ( 0.000,-0.000, 1.000) through (0.333333,0.333333,0.000000)
 2  axis along ( 0.000, 0.000, 1.000) through (0.333333,0.333333,0.000000)
 3  axis along (-0.000, 0.000, 1.000) through (0.666667,0.666667,0.000000)
 6  axis along ( 0.000, 0.000, 1.000) through (0.333333,0.333333,0.000000)
-2  axis along ( 2.000,-1.000, 0.000) through (0.333333,0.333333,0.250000)
-2  axis along ( 1.000, 0.000, 0.000) through (0,0,0)
-2  axis along ( 1.000, 1.000, 0.000) through (0.333333,0.333333,0.250000)
-2  axis along ( 0.000, 1.000, 0.000) through (0.000000,0.500000,0.000000)
-2  axis along (-1.000, 2.000, 0.000) through (0.333333,0.333333,0.250000)
-2  axis along ( 1.000,-1.000, 0.000) through (0.000000,0.500000,0.000000)
EOF
npass out.dat out2.dat
fi

echo -n 'Symmetry ops from castep_bin file and conversion: '
$c2x -v --symmetry --cell SiC_4H.cell 2>/dev/null | grep '^!' | \
    grep -v axis | sort > out.dat
$c2x -v --cell SiC_4H.castep_bin 2>/dev/null | grep '^!' | \
    grep -v axis | sort > out2.dat
pass out.dat out2.dat

rm out.cell out2.cell

echo
echo "SPG tests complete."
echo "Passed: $passes"
echo "Failed: $fails"
echo

total_passes=$((total_passes+passes))
total_fails=$((total_fails+fails))

passes=0
fails=0

# Read atoms from check files. Use format which will not record
# forces or pseudopotential blocks

echo -n 'Read atoms from check file (v16.11): '
$c2x --shelx Li_16.11.check out1.res
$c2x --shelx Li.cell out2.res
npass out1.res out2.res

# Compare charge density to reconstruction from wavefunction
# Remove the different titles
# Note that this example tests weighting of full and empty bands
# and different kpt weights

# The following needs norm conserving pseudopots
echo -n 'Read charge density from check file vs reconstruction from bands: '
$c2x -c Li_16.11.check | sed -e '/BEGIN_DATAGRID_3D/d' -e '/^#/d' > out1.xsf
$c2x -BAW -k=- Li_16.11.check | sed -e '/BEGIN_DATAGRID_3D/d' -e '/^#/d' > out2.xsf
npass out1.xsf out2.xsf

# Repeat for 6.1 format check files

if [ -e Li_6.1.check ]
then
    echo -n 'Read atoms from check file (v6.1): '
    $c2x --shelx Li_6.1.check out1.res
    $c2x --shelx Li.cell out2.res
    npass out1.res out2.res

# Remember Castep 6.1 will not do OTF generation of norm conserving pots

    echo -n 'Read charge density from check file (v6.1): '
    $c2x -c Li_6.1.check | sed -e '/BEGIN_DATAGRID_3D/d' -e '/^#/d' > out1.xsf
    $c2x -BAW -k=- Li_6.1.check | sed -e '/BEGIN_DATAGRID_3D/d' -e '/^#/d' > out2.xsf
    npass out1.xsf out2.xsf
fi
rm out1.res out2.res out1.xsf out2.xsf

echo -n 'Read charge density from PWscf file vs reconstruction from bands: '
$c2x -c Li.save/data-file-schema.xml | sed -e '/BEGIN_DATAGRID_3D/d' -e '/^#/d' > out1.xsf
$c2x -BAW -k=- Li.save/data-file-schema.xml | sed -e '/BEGIN_DATAGRID_3D/d' -e '/^#/d' > out2.xsf
npass out1.xsf out2.xsf
rm out1.xsf out2.xsf

# Need to run abinit with spgroup 1 for this
echo -n 'Read charge density from Abinit file vs reconstruction from bands: '
$c2x -c Li_DEN | sed -e '/BEGIN_DATAGRID_3D/d' -e '/^#/d' > out1.xsf
$c2x -BAW -k=- Li_WFK | sed -e '/BEGIN_DATAGRID_3D/d' -e '/^#/d' > out2.xsf
npass out1.xsf out2.xsf
rm out1.xsf out2.xsf

echo -n 'Read header from Abinit binary file: '
$c2x --cell_abc Li_DEN out1.cell
head -15 Li.cell > out2.cell
npass out1.cell out2.cell
rm out1.cell out2.cell

echo -n 'Checking VASP writing and re-reading: '

$c2x --vasp -15 -c HCl.check HCl.CHGCAR
$c2x --cube -c HCl.check | sed -e '/^1 /d' -e '/^17 /d' > HCl.cube
$c2x --cube -c HCl.CHGCAR | sed -e '/^1 /d' -e '/^17 /d' > out.cube
npass HCl.cube out.cube
rm HCl.cube out.cube

echo -n 'Checking VASP reading and denfmt writing: '
$c2x --denfmt -c HCl.CHGCAR out1.dat
$c2x --denfmt -c HCl.check out2.dat
npass out1.dat out2.dat
rm out1.dat out2.dat HCl.CHGCAR

echo -n 'Checking cube reading/writing and density interpolation: '

$c2x -c -15 -i=64,72,75 --cube HCl.check out.cube
$c2x -c -i=45,40,32 --cube out.cube out2.cube
$c2x -c -i=45,40,32 --cube HCl.check out.cube
npass out.cube out2.cube
rm out.cube out2.cube

echo -n 'Further check of interpolation with transforms: '
$c2x -c -x='(1,0,0)(0,2,0)(0,0,3)' HCl.check out.xsf
$c2x -c -Q -x=1x2x3 HCl.check out2.xsf
npass out.xsf out2.xsf
rm out.xsf out2.xsf

echo -n 'Checking line interpolation: '
$c2x -c -P=H1:Cl1:9 HCl.check | sed -e '/^#/d' -e '/^$/d' > out.dat
npass out.dat HCl.dat
echo -n 'Checking line interpolation (2): '
$c2x -c -P='(.0142857,.33333333,.6):(.1957143,.33333333,.6):9' HCl.check | \
    sed -e '/^#/d' -e '/^$/d' > out.dat
npass out.dat HCl.dat
echo -n 'Checking line interpolation (3): '
$c2x -c -P=H:Cl:9 HCl.check | sed -e '/^#/d' -e '/^$/d' > out.dat
npass out.dat HCl.dat

echo -n 'Checking point interpolation: '
$c2x -c -z=Cl HCl.check | sed 's/^.*: //' > out1.dat
echo 0.35408 > out2.dat
npass out1.dat out2.dat

echo -n 'Checking point interpolation (2): '
$c2x -c -z=O1 FeO.castep_bin 2>&1 > out1.dat
$c2x -c -z=O2 FeO.castep_bin 2>&1 > out2.dat
x=`cat out1.dat`
x=${x##*:}
echo $x > out1.dat
x=`cat out2.dat`
x=${x##*:}
echo $x > out2.dat
tol="$tol2"
npass out1.dat out2.dat
tol="$tol1"

rm out1.dat out2.dat out.dat


echo -n 'Checking spin density extraction and .castep_bin reading: '
$c2x -vs --null FeO.castep_bin 2>&1 | grep 'min=' > out1.dat
echo '  min=-8.45041 max=8.45102 sum=0.119667 int=0.00030401 int|s|=7.0031' > out2.dat
npass out1.dat out2.dat


# Check that the sum of the integrals of two spinor components of
# a single band is one

echo -n 'Checking spinor extraction: '
$c2x -B=1 -S=0 --null -v GaAs.check 2>&1 | grep 'min=' > out1.dat
$c2x -B=1 -S=1 --null -v GaAs.check 2>&1 | grep 'min=' > out2.dat
x=`cat out1.dat`
x=${x##*int=}
y=`cat out2.dat`
y=${y##*int=}
echo $x + $y | bc -q > out1.dat
echo 1 > out2.dat
npass out1.dat out2.dat
rm out1.dat out2.dat

echo
echo "Density tests complete."
echo "Passed: $passes"
echo "Failed: $fails"
echo

total_passes=$((total_passes+passes))
total_fails=$((total_fails+fails))

echo "Total score: passed $total_passes"
echo "             failed $total_fails"
